/*
 *  Searching/Queries/Rating.cs
 *  Copyright (C) 2006-2007 Marcin Krystian Krzywonos
 *  License: GNU/GPL version 2
 */


using System;
using System.Collections;
using System.Xml;
using Mono.Unix;
using Incollector.Engine;
using Incollector.GUI.Widgets;


namespace Incollector.Searching.Queries {
	
	public class Rating : Incollector.Searching.Query {
		
		protected RatingEntry  rating;
		protected Gtk.Widget   widget;
		protected Gtk.ComboBox conditionals;
		
		public enum ConditionalEnum {
			Equal          = 0,
			LessOrEqual    = 1,
			GreaterOrEqual = 2
		}
		
		protected ConditionalEnum Conditional {
			get { return (ConditionalEnum) conditionals.Active; }
			set { conditionals.Active = (int) value; }
		}
		
		protected int RatingValue {
			get { return this.rating.Rating;  }
			set { this.rating.Rating = value; }
		}
		
		public override Gtk.Widget QueryWidget {
			get { return widget; }
		}
		
		public override string ID {
			get { return "rating"; }
		}
		
		public override string Name {
			get { return Catalog.GetString ("Rating"); }
		}
		
		
		public Rating ()
		{
			widget = new Gtk.HBox ();
			(widget as Gtk.HBox).Spacing = 6;
			
			conditionals = new Gtk.ComboBox (new string [] {
								Catalog.GetString ("is equal"),
								Catalog.GetString ("is less or equal"),
								Catalog.GetString ("is greater or equal")
							});
			conditionals.Active = 0;
			(widget as Gtk.HBox).PackStart (conditionals, true, true, 0);
			
			rating = new RatingEntry ();
			rating.Rating = 0;
			(widget as Gtk.HBox).PackStart (rating, false, false, 0);
			
			widget.ShowAll ();
		}
		
		
		public override Query New ()
		{
			return (Query) new Rating ();
		}
		
		
		public override Query Clone ()
		{
			Rating query = new Rating ();
			query.RatingValue = this.RatingValue;
			query.Conditional = this.Conditional;
			
			return query;
		}
		
		
		public override bool CheckQuery (Engine.Entry entry)
		{
			switch (Conditional) {
				case ConditionalEnum.Equal:
					return entry.Rating == RatingValue;
				
				case ConditionalEnum.LessOrEqual:
					return entry.Rating <= RatingValue;
				
				case ConditionalEnum.GreaterOrEqual:
					return entry.Rating >= RatingValue;
			}
			
			return true;
		}

		
		public override Query ReadXml (XmlTextReader reader)
		{
			Rating query = new Rating ();
			
			query.Conditional = (ConditionalEnum) System.Enum.Parse (typeof (ConditionalEnum), reader.GetAttribute ("conditional"));
			
			try {
				query.RatingValue = Convert.ToInt32 (reader.GetAttribute ("rating"));
			} catch {
				query.RatingValue = 0;
			}
			
			return query;
		}
		
		
		public override void WriteXml (XmlTextWriter writer)
		{
			writer.WriteStartElement (this.ID);
				writer.WriteAttributeString ("conditional", Conditional.ToString ());
				writer.WriteAttributeString ("rating", RatingValue.ToString ());
			writer.WriteEndElement ();
		}
	}
}
